#if (!requireNamespace("BiocManager", quietly = TRUE))
#    install.packages("BiocManager")
#BiocManager::install(c("car", "ridge", "preprocessCore", "genefilter", "sva"))

#install.packages("ggpubr")
rm(list = ls())
library(limma)
library(ggpubr)
library(reshape2)
library(pRRophetic)
library(ggplot2)
library(ggExtra)
pFilter=0.05                #
expFile="merge.txt"         #
scoreFile="risk.all.txt"     
data(cgp2016ExprRma)
data(PANCANCER_IC_Tue_Aug_9_15_28_57_2016)
drugData2016 =drugData2016[,drugData2016$Tissue.sub.type = "ovary"] 
allDrugs=unique(drugData2016$Drug.name)

rt =data.table::fread (expFile, header=T, sep="\t", check.names=F)
rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
data=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
data=avereps(data)
data=data[rowMeans(data)>0,]
data=t(data)
rownames(data)=gsub("(.*?)\\_(.*?)", "\\2", rownames(data))
data=t(data)

riskRT=read.table("risk.all.txt", header=T, sep="\t", check.names=F, row.names=1)
riskRT$riskScore[riskRT$riskScore>quantile(riskRT$riskScore,0.99)]=quantile(riskRT$riskScore,0.99)

outTab=data.frame()
sigDrugs=c()
for(drug in allDrugs){

	possibleError=tryCatch(
    	{senstivity=pRRopheticPredict(data, drug, selection=1, dataset = "cgp2016")},
    	error=function(e) e)
    if(inherits(possibleError, "error")){next}
	senstivity=senstivity[senstivity!="NaN"]
	senstivity[senstivity>quantile(senstivity,0.99)]=quantile(senstivity,0.99)
	
	sameSample=intersect(row.names(riskRT), names(senstivity))
	risk=riskRT[sameSample, c("riskScore","risk"),drop=F]
	senstivity=senstivity[sameSample]
	rt=cbind(risk, senstivity)
	
	rt$risk=factor(rt$risk, levels=c("low", "high"))
	type=levels(factor(rt[,"risk"]))
	comp=combn(type, 2)
	my_comparisons=list()
	for(i in 1:ncol(comp)){my_comparisons[[i]]<-comp[,i]}

	test=wilcox.test(senstivity~risk, data=rt)
	diffPvalue=test$p.value

	x=as.numeric(rt[,"riskScore"])
	y=as.numeric(rt[,"senstivity"])
	corT=cor.test(x, y, method="spearman")
	corPvalue=corT$p.value
	corR = corT[["estimate"]][["rho"]]
	
	if((diffPvalue<pFilter) & (corPvalue<pFilter)){
		boxplot=ggboxplot(rt, x="risk", y="senstivity", fill="risk",
					      xlab="risk",
					      #add = "jitter",
					      color = "black",
					      ylab=paste0(drug, " senstivity (IC50)"),
					      legend.title="risk",
					      palette=c("springgreen3","orangered3")
					     )+ 
			stat_compare_means(comparisons=my_comparisons)
		pdf(file=paste0("durgSenstivity.", drug,  ".pdf"), width=3.5, height=4)
		print(boxplot)
		dev.off()
		df1=as.data.frame(cbind(x,y))
		p1=ggplot(df1, aes(x, y)) + 
				xlab("Risk score") + ylab(paste0(drug, " senstivity (IC50)"))+
				geom_point() + geom_smooth(method="lm",formula = y ~ x) + theme_bw()+
				stat_cor(method = 'spearman', aes(x =x, y =y))
		p2=ggMarginal(p1, type="density", xparams=list(fill = "palevioletred1"), yparams=list(fill = "steelblue2"))
		
		pdf(file=paste0("Cor.", drug, ".pdf"), width=4 , height=4 )
		print(p2)
		dev.off()
		
		drug1 = drug
		
		#record
		sigDrugs = c(sigDrugs, drug1)
		diffPvalue1 =  diffPvalue
		corPvalue1 = corPvalue
		R1 = corR
		y_high_rt = as.numeric(rt[rt$risk == "high",][,"senstivity"]) 
		y_low_rt  = as.numeric(rt[rt$risk == "low", ][,"senstivity"])  
		IC50value_high = median(y_high_rt)
		SDvalue_high   = sd(y_high_rt)
		IC50value_low = median(y_low_rt)
		SDvalue_low   = sd(y_low_rt)
		
		
		outTab = rbind(outTab,cbind(drugName=drug1,
		                            Sensitivity_Pvalue=diffPvalue1,
		                            cor_Pvalue=corPvalue1,
		                            R = R1,
		                            IC50median_low = IC50value_low,
		                            SD_low        = SDvalue_low,
		                            IC50median_high = IC50value_high,
		                            SD_high       = SDvalue_high))
	}
}


class(outTab)

write.table(outTab,file="DrugResponse.txt",sep="\t",row.names=F,quote=F)

